import __fiber__
import unittest
from test import test_support


class TestSimpleGenerators(unittest.TestCase):
    def test_simple_gen(self):
        def f():
            yield 1
            yield 2
        g = f()
        self.assertEqual([1,2],list(g))
        self.assertRaises(StopIteration, g.next)

    def test_gen_which_returns(self):
        def f():
            yield 1
            return
            yield 2
        self.assertEqual([1],list(f()))

    def test_gen_which_raises_StopIteration(self):
        def f():
            yield 1
            raise StopIteration
            yield 2
        self.assertEqual([1],list(f()))

    def test_gen_which_return_vs_catch_StopIteration(self):
        def f():
            try:
                return
            except:
                yield 1
        self.assertEqual([],list(f()))
        def f():
            try:
                raise StopIteration
            except:
                yield 1
        self.assertEqual([1],list(f()))

    def test_return_finally(self):
        def f():
            try:
                return
            finally:
                yield 1
        self.assertEqual([1],list(f()))

    def test_try_except_finally(self):
        def f():
            try:
                yield 1
                try:
                    yield 2
                    1//0
                    yield 3  # never get here
                except ZeroDivisionError:
                    yield 4
                    yield 5
                    raise
                except:
                    yield 6
                yield 7     # the "raise" above stops this
            except:
                yield 8
            yield 9
            try:
                x = 12
            finally:
                yield 10
            yield 11
        self.assertEqual([1, 2, 4, 5, 8, 9, 10, 11],list(f()))

class TestEnhancedGenerators(unittest.TestCase):
    def test_send(self):
        def f():
            x = yield
            yield x
        g = f()
        g.next()
        self.assertEqual(g.send(1), 1)
        self.assertRaises(StopIteration, g.next)

    def test_yield_with_parens(self):
        def f():
            x = (yield)
            (yield x)
        g = f()
        g.next()
        self.assertEqual(g.send(1), 1)
        self.assertRaises(StopIteration, g.next)

    def test_yield_plus(self):
        def f():
            x = (yield 2)+12
            (yield x)
        g = f()
        self.assertRaises(TypeError, g.send, 1)
        self.assertEqual(g.next(), 2)
        self.assertEqual(g.send(1), 13)
        self.assertRaises(StopIteration, g.next)

    def test_no_nesting(self):
        target = """
        def f():
            x = (yield 2)+(yield 3)
        """
        target = __fiber__.prepare_source(target)
        from EasyExtend.eecompiler import EECompiler
        eec = EECompiler(__fiber__)
        self.assertRaises(NotImplementedError, eec.eecompile_suite, target)

    def test_red_stream(self):
        def reduce_stream(func, strm):
            res = None
            while 1:
                try:
                    try:
                        if res is None:
                            res, v = strm[:2]
                            del strm[0:2]
                        else:
                            v = strm[0]
                            del strm[0]
                    except (ValueError, IndexError):
                        l = yield
                    else:
                        out = func(res, v)
                        res = out if out is not None else res
                        l = yield res
                    if l:
                        strm.extend(l)
                        yield
                except YieldResult:
                    yield res

        def reduced(red_stream):
            res = red_stream.next()
            if res is None:
                return red_stream.throw(YieldResult)
            for item in red_stream:
                if item is None:
                    break
                else:
                    res = item
            return res

        stm = reduce_stream(lambda x,y: x+y, [1,2,3])
        self.assertEqual(stm.next(), 3)
        self.assertEqual(stm.next(), 6)
        self.assertEqual(stm.next(), None)
        stm.send([4,5,6])
        self.assertEqual(stm.next(), 10)
        self.assertEqual(reduced(stm), 21)



def test_main():
    test_support.run_unittest(TestSimpleGenerators)
    test_support.run_unittest(TestEnhancedGenerators)


if __name__ == "__main__":
    test_main()



